/*
  Working Prototype: Average of Dice Games
  James Kobayashi

  This program is set to run a game of 12 different sets of rolls.
  
  On start of the program, the first set of rolls will begin.
  The program will generate a random number between 1 and 6.
  One turn consists of 1 roll. If the roll equals 1, the total
  score results in a 0, otherwise the total score is saved.
  This is one turn. One roll will perform 100,000 turns.
  
  When 100,000 turns have been passed, the second set of rolls
  follows. Each turn will then attempt two rolls. Turns that 
  end prematurely by rolling a 1 saves a total of 0 otherwise
  the total score is saved. Again, the second roll will 
  perform 100,000 turns.
  
  12 sets of rolls is done, 100,000 turns each.
  Then the average of the rolls of all the games is shown.

  The purpose of this prototype is to compare the optimal
  number of rolls as suggested by the steady state vector
  with the averages produce by this program. 

  The steady state vector suggests that the probability of
  rolling a number not equal to 1 is about 85.71% or 6
  out of 7 rolls. The program will demonstrate the game
  and compute the averages of a player rolling 1 - 12 rolls.
*/

#include<iostream>
#include<stdlib.h>
#include<stdio.h>
#include<time.h>

// Number of set of rolls to take per turn
#define ROLLS 12

// Number of turns to take per set of rolls
#define TRIALS 100000

using namespace std;

int main( ) 
{
  srand(time(NULL));

  // Dataset to hold the averages of the current game
  int scoreset[ROLLS][TRIALS];
  
  // For each roll, 
  // first iteration = one roll per turn
  // second iteration = two rolls, etc
  for(int i = 0; i < ROLLS; i++)
    {
      
      // For every roll, take 100,000 turns
      // Every iteration is one roll
      for(int j = 0; j < TRIALS; j++)
	{
	  
	  // Begin the turn with 0 values
	  int points = 0;
	  int score = 0;
	      
	  // i = number of rolls to roll per turn
	  for(int k = 0; k <= i; k++)
	    {
	      points = rand( ) % 6 + 1;
	      score = score + points;
	      
	      // If the roll = 1, end the turn
	      // Set the turn's score to 0
	      if(points == 1)
		{
		  k = i;
		  score = 0;
		}
	    }
	  
	  scoreset[i][j] = score;
	}
    }

  // Game has ended, averages and output begin now


  // Dataset to hold the average each set of rolls
  double rollset[ROLLS];
  double averagescore = 0.0;
  
  // Average all 10000 turns for each roll
  for(int i = 0; i < ROLLS; i++)
    {
      for(int j = 0; j < TRIALS; j++)
	averagescore = averagescore + scoreset[i][j];
      
      averagescore = averagescore / TRIALS;
      
      rollset[i] = averagescore;
    }
              
  cout << "Total Roll Averages" << endl;
  for(int i = 0; i < ROLLS; i++)
    {
      cout << "Roll(" << i + 1 << "): " << rollset[i] << endl;
    }
  cout << endl;

  
  // Output the average number of turns points were scored
  cout << "Averages from set 5:" << endl;
  double turnaverage = 0.0;
  double mean = 0.0;
  for(int i = 0; i < TRIALS; i++)
    {
      if(scoreset[4][i] != 0)
	{
	  mean = mean + scoreset[4][i];
	  turnaverage++;
	}
    }
    
  mean = mean / turnaverage;
  cout << "Mean of Successful Turn's points: " << mean << endl;
  cout << "Probability of a Successful Turn: " << turnaverage/TRIALS << endl;;

  cout << endl << "Averages of set 6:" << endl;
  turnaverage = 0.0;
  mean = 0.0;
  for(int i = 0; i < TRIALS; i++)
    {
      if(scoreset[5][i] != 0)
	{
	  mean = mean + scoreset[5][i];
	  turnaverage++;
	}
    }
    
  mean = mean / turnaverage;
  cout << "Mean of Successful Turn's points: " << mean << endl;
  cout << "Probability of a Successful Turn: " << turnaverage/TRIALS << endl << endl;


  /*
  // Output the number of points for every turn
  cout << "Rolls from set 5:" << endl;
  for(int i = 0; i < TRIALS; i++)
    {
      for(int j = 0; j < 25; j++)
	{
	  cout << scoreset[4][i] << " ";
	  i++;
	}
      cout << endl;
    }
  cout << endl;
  */

  return 0;
}

